import os
from aiohttp import web
from configparser import ConfigParser

from singleton import Singleton
from services.basicservice import BasicService


class AnorGo(BasicService, metaclass=Singleton):
    def __init__(self):
        super().__init__(__file__)
        conf = ConfigParser()
        conf.read(os.path.join(self.serviceDir, 'config.ini'))
        self.servicePath = conf.get('anorgo', 'path') if conf.has_option('anorgo', 'path') else '/api/v1'
        self._loadCases('cases.json')

    def getRoutes(self) -> list[web.AbstractRouteDef]:
        return [web.post(os.path.join(self.servicePath, 'payment'), handler=self.handler),
                web.get(os.path.join(self.servicePath, 'payment/status'), handler=self.handler),
                web.delete(os.path.join(self.servicePath, 'payment/reversal'), handler=self.handler),
                web.post(os.path.join(self.servicePath, 'payment/ofd_data/submit_qrcode'), handler=self.handler)]

    async def handler(self, request: web.Request) -> web.Response:
        self.log.info(f"Данные запроса '{await request.json()}'")
        return self._createResponse(request.path)


instance = AnorGo()
