from aiohttp import web

from singleton import Singleton
from services.basicservice import BasicService


class ArtixBonus(BasicService, metaclass=Singleton):
    def __init__(self):
        super().__init__(__file__)
        self._loadCases('cases.json')
    
    def getRoutes(self) -> list[web.AbstractRouteDef]:
        return [web.get('/ACC/rest/v1/cards/{cardNumber}', handler=self.cardByNumber),
                web.post('/ACC/rest/v1/assets', handler=self.transaction),
                web.post('/ACC/rest/v1/assets/back', handler=self.transactionBack),
                web.put('/ACC/rest/v1/assets/{sessionId}/commit', handler=self.transactionCommit),
                web.put('/ACC/rest/v1/assets/{sessionId}/rollback', handler=self.transactionRollback)]

    async def cardByNumber(self, request: web.Request) -> web.Response:
        self.log.info(f"Запрос данных о карте по номеру карты: '{request.match_info['cardNumber']}'")
        self._loadCases('cases.json')
        return self._createResponse(request.path if self._hasCases(request.path) else '/ACC/rest/v1/cards/<cardNumber>')

    async def transaction(self, request: web.Request) -> web.Response:
        self.log.info(f"Провести операцию: данные транзакции: '{request.query_string}'")
        return self._createResponse(request.path if self._hasCases(request.path) else '/ACC/rest/v1/assets<transactionData>')
  
    async def transactionBack(self, request: web.Request) -> web.Response:
        self.log.info(f"Провести операцию отмены: данные транзакции: '{request.query_string}'")
        return self._createResponse(request.path if self._hasCases(request.path) else '/ACC/rest/v1/assets/back<transactionData>')
  
    async def transactionRollback(self, request: web.Request) -> web.Response:
        self.log.info(f"Закрыть операцию '{request.match_info['sessionId']}' отменой её воздействия: данные транзакции: '{request.query_string}'")
        return self._createResponse(request.path if self._hasCases(request.path) else '/ACC/rest/v1/assets/<sessionId>/rollback<transactionData>')

    async def transactionCommit(self, request: web.Request) -> web.Response:
        self.log.info(f"Закрыть операцию '{request.match_info['sessionId']}' потдверждением её воздействия: '{request.query_string}'")
        return self._createResponse(request.path if self._hasCases(request.path) else '/ACC/rest/v1/assets/<sessionId>/commit<transactionData>')


instance = ArtixBonus()
