import xml.etree.ElementTree as ET
from typing import List
from aiohttp import web

from singleton import Singleton
from services.basicservice import BasicService


class Egais(BasicService, metaclass=Singleton):
    exciseBottleAttrs = ['price', 'barcode', 'ean', 'volume']

    def __init__(self):
        super().__init__(__file__)
        self._loadCases("cases.json")

    def getRoutes(self) -> list[web.AbstractRouteDef]:
        return [web.get('/', handler=self.checkServerAvailable),
                web.post('/xml', handler=self.getEgaisCheck)]

    async def checkServerAvailable(self, request: web.Request) -> web.Response:
        self.log.info('Запрос доступности сервера')
        return self._createResponse(request.path)

    async def getEgaisCheck(self, request: web.Request) -> web.Response:
        self.log.info('Получаем данные для ЕГАИС')
        reader = await request.multipart()
        xml = ""
        while True:
            part = await reader.next() 
            if part.name == "xml_file":
                xml = await part.text()
                break
        try:
            root = ET.fromstring(xml)
            excise = root.findall('Bottle')
            if len(excise) > 0:
                self.log.info('Проверка акцизной продукции')
                Egais.checkExcise(excise)
        except Exception as e:
            self.log.error(e)
            message = 'Некорректные данные чека: cheque.xml'
            self.log.warning(message)
            raise web.HTTPBadRequest(text=message)

        self.log.info('Отправляем ответ от ЕГАИС')
        return self._createResponse(request.path)

    @staticmethod
    def checkExcise(xmlNodesList: List[ET.Element]) -> None:
        """ Проверка акцизной продукции на наличие обязательных полей
        Проверка наличия значений и типов не производится,  кроме проверки уникальности поля 'barcode'
        """
        barcodes = {}
        for xmlNode in xmlNodesList:
            for attr in Egais.exciseBottleAttrs:
                if xmlNode.attrib.get(attr) is None:
                    raise web.HTTPBadRequest(text=f'В акцизной продукции {xmlNode} отсутствует необходимый атрибут: {attr}')
                if attr == 'barcode':
                    barcodes[xmlNode.attrib.get(attr)] = 1
        if len(xmlNodesList) != len(barcodes):
            raise web.HTTPBadRequest(text='Обнаружено дублирование акцизной продукции')


instance = Egais()
