import os
from aiohttp import web

from configparser import ConfigParser
from services.basicservice import BasicService
from singleton import Singleton


class Iturup(BasicService, metaclass=Singleton):
    def __init__(self):
        super().__init__(__file__)
        conf = ConfigParser()
        conf.read(os.path.join(self.serviceDir, 'config.ini'))
        self.servicePath = conf.get('iturup', 'path') if conf.has_option('iturup', 'path') else '/iturup'
        self._loadCases('cases.json')

    def getRoutes(self) -> list[web.AbstractRouteDef]:
        return [web.post(os.path.join(self.servicePath, 'pay'), handler=self.pay),
                web.post(os.path.join(self.servicePath, 'send_check'), handler=self.sendCheck)]

    async def pay(self, request: web.Request) -> web.Response:
        self.log.info("Проверка баланса и оплата товара")
        return self._createResponse(request.path)

    async def sendCheck(self, request: web.Request) -> web.Response:
        self.log.info("Отправка чека")
        return self._createResponse(request.path)

instance = Iturup()
