from aiohttp import web

from services.basicservice import BasicService
from singleton import Singleton


class RaiffeisenSbp(BasicService, metaclass=Singleton):
    def __init__(self):
        super().__init__(__file__)
        self._loadCases('raiffeisensbp.json')

    def getRoutes(self) -> list[web.AbstractRouteDef]:
        return [web.post('/payment/v1/orders', handler=self.orders),
                web.get('/payment/v1/orders/{orderId}', handler=self.ordersStatus),
                web.post('/payments/v1/orders/{orderId}/refunds/{refundId}', handler=self.refunds),
                web.get('/payments/v1/orders/{orderId}/refunds/{refundId}', handler=self.refundsStatus),
                web.get('/sbp/v2/qrs/{qrId}', handler=self.getQrRegistrationData),
                web.delete('/sbp/v2/qrs/{qrId}', handler=self.cancelQR),
                web.post('/sbp/v2/qrs', handler=self.qrRegistration),
                web.get('/sbp/v1/qr/{qrId}/payment-info', handler=self.getQrData),
                web.post('/sbp/v1/refund', handler=self.refund),
                web.get('/sbp/v1/refund/{refundId}', handler=self.getRefundStatus)]

    async def qrRegistration(self, request: web.Request) -> web.Response:
        self.log.info("Регистрация QR")
        return self._createResponse(request.path)

    async def getQrData(self, request: web.Request) -> web.Response:
        self.log.info("Получение данных по зарегистрированному QR")
        return self._createResponse('/sbp/v1/qr/<qrId>/payment-info')

    async def getQrRegistrationData(self, request: web.Request) -> web.Response:
        self.log.info("Получение данных по зарегистрированному QR")
        return self._createResponse('/sbp/v2/qrs/<qrId>', request.method)

    async def refund(self, request: web.Request) -> web.Response:
        self.log.info("Оформление возврата по платежу")
        return self._createResponse(request.path)

    async def getRefundStatus(self, request: web.Request) -> web.Response:
        self.log.info("Получение информации по возврату")
        return self._createResponse('/sbp/v1/refund/<refundId>')

    async def cancelQR(self, request: web.Request) -> web.Response:
        self.log.info("Отмена QR")
        return self._createResponse('/sbp/v2/qrs/<qrId>', request.method)

    async def orders(self, request: web.Request) -> web.Response:
        self.log.info("Регистрация заказа")
        return self._createResponse(request.path)

    async def ordersStatus(self, request: web.Request) -> web.Response:
        self.log.info("Получение статуса заказа")
        return self._createResponse('/payment/v1/orders/<orderId>')

    async def refunds(self, request: web.Request) -> web.Response:
        self.log.info("Отмена заказа")
        return self._createResponse('/payments/v1/orders/<orderId>/refunds/<refundId>', request.method)

    async def refundsStatus(self, request: web.Request) -> web.Response:
        self.log.info("Получение статуса отмены")
        return self._createResponse('/payments/v1/orders/<orderId>/refunds/<refundId>', request.method)


instance = RaiffeisenSbp()
