import os
from aiohttp import web

from services.basicservice import BasicService
from singleton import Singleton
from configparser import ConfigParser


class SmVision(BasicService, metaclass=Singleton):
    def __init__(self):
        super().__init__(__file__)
        conf = ConfigParser()
        conf.read(os.path.join(self.serviceDir, 'config.ini'))
        self.servicePath = conf.get('smvision', 'path') if conf.has_option('smvision', 'path') else '/smvision'
        self._loadCases('smvision.json')

    def getRoutes(self) -> list[web.AbstractRouteDef]:
        return [web.post(os.path.join(self.servicePath, 'recognize/{partnerId}'), handler=self.recognize),
                web.post(os.path.join(self.servicePath, 'upload'), handler=self.upload)]

    async def recognize(self, request: web.Request) -> web.Response:
        self.log.info('Распознание товара')
        return self._createResponse(request.path if self._hasCases(request.path) else 'recognize/<partnerId>')

    async def upload(self, request: web.Request) -> web.Response:
        self.log.info('Обновление данных для распознания товара')
        return self._createResponse(request.path)


instance = SmVision()
