import os
from aiohttp import web
from services.basicservice import BasicService

from singleton import Singleton
from configparser import ConfigParser


class Uds(BasicService, metaclass=Singleton):
    def __init__(self):
        super().__init__(__file__)
        conf = ConfigParser()
        conf.read(os.path.join(self.serviceDir, 'config.ini'))
        self.servicePath = conf.get('uds', 'path') if conf.has_option('uds', 'path') else '/uds'
        self._loadCases('cases.json')
    
    def getRoutes(self) -> list[web.AbstractRouteDef]:
        return [web.get(os.path.join(self.servicePath, 'tags'), handler=self.tags),
                web.get(os.path.join(self.servicePath, 'settings'), handler=self.settings),
                web.get(os.path.join(self.servicePath, 'customers/find'), handler=self.findCustomer),
                web.post(os.path.join(self.servicePath, 'operations/calc'), handler=self.calculate),
                web.post(os.path.join(self.servicePath, 'operations'), handler=self.operate),
                web.post(os.path.join(self.servicePath, 'operations/{operationId}/refund'), handler=self.refund)]

    async def tags(self, request: web.Request) -> web.Response:
        self.log.info("Получение тэгов компании (проверка связи)")
        return self._createResponse(request.path)

    async def settings(self, request: web.Request) -> web.Response:
        self.log.info("Получение настроек компании")
        return self._createResponse(request.path)

    async def findCustomer(self, request: web.Request) -> web.Response:
        self.log.info(f"Поиск клиента с параметрами: '{request.query_string}'")
        return self._createResponse(request.path)

    async def calculate(self, request: web.Request) -> web.Response:
        self.log.info(f"Информация о доступных бонусных баллах для списания или размере скидки. Данные запроса: '{await request.json()}'")
        return self._createResponse(request.path)

    async def operate(self, request: web.Request) -> web.Response:
        self.log.info(f"Проведение операции. Данные запроса: '{await request.json()}'")
        return self._createResponse(request.path)

    async def refund(self, request: web.Request) -> web.Response:
        self.log.info(f"Возврат по операции '{request.match_info['operationId']}'")
        return self._createResponse(request.path if self._hasCases(request.path) else 'operations/<operationId>/refund')

instance = Uds()
